@@ -9,7 +9,7 @@
 */
 
 /* ====================================================================
- * Copyright (c) 1998-1999 Ralf S. Engelschall. All rights reserved.
+ * Copyright (c) 1998-2000 Ralf S. Engelschall. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -338,13 +338,17 @@ void ssl_init_TmpKeysHandle(int action, server_rec *s, pool *p)
     unsigned char *ucp;
     RSA *rsa;
     DH *dh;
+    int n;
 
     /* Generate Keys and Params */
     if (action == SSL_TKP_GEN) {
 
-        ssl_log(s, SSL_LOG_INFO, "Init: Generating temporary RSA private keys (512/1024 bits)");
+        /* seed PRNG */
+        n = ssl_rand_seed(s, p, SSL_RSCTX_STARTUP);
+        ssl_log(s, SSL_LOG_INFO, "Init: Seeding PRNG with %d bytes of entropy", n);
 
         /* generate 512 bit RSA key */
+        ssl_log(s, SSL_LOG_INFO, "Init: Generating temporary RSA private keys (512/1024 bits)");
         if ((rsa = RSA_generate_key(512, RSA_F4, NULL, NULL)) == NULL) {
             ssl_log(s, SSL_LOG_ERROR, "Init: Failed to generate temporary 512 bit RSA private key");
             ssl_die();
@@ -540,7 +544,7 @@ void ssl_init_ConfigureServer(server_rec *s, pool *p, SSLSrvConfigRec *sc)
      * Configure additional context ingredients
      */
     SSL_CTX_set_options(ctx, SSL_OP_SINGLE_DH_USE);
-    if (mc->nSessionCacheMode == SSL_SCMODE_UNSET)
+    if (mc->nSessionCacheMode == SSL_SCMODE_NONE)
         SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_OFF);
     else
         SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_SERVER);
@@ -704,7 +708,14 @@ void ssl_init_ConfigureServer(server_rec *s, pool *p, SSLSrvConfigRec *sc)
                         cpVHostID, (i == SSL_AIDX_RSA ? "RSA" : "DSA"), pathlen);
             }
             if (SSL_X509_getCN(p, sc->pPublicCert[i], &cp)) {
-                if (strNE(s->server_hostname, cp)) {
+                if (ap_is_fnmatch(cp) &&
+                    !ap_fnmatch(cp, s->server_hostname, FNM_PERIOD|FNM_CASE_BLIND)) {
+                    ssl_log(s, SSL_LOG_WARN,
+                        "Init: (%s) %s server certificate wildcard CommonName (CN) `%s' "
+                        "does NOT match server name!?", cpVHostID, 
+                        (i == SSL_AIDX_RSA ? "RSA" : "DSA"), cp);
+                }
+                else if (strNE(s->server_hostname, cp)) {
                     ssl_log(s, SSL_LOG_WARN,
                         "Init: (%s) %s server certificate CommonName (CN) `%s' "
                         "does NOT match server name!?", cpVHostID, 
@@ -873,7 +884,7 @@ void ssl_init_CheckServers(server_rec *sm, pool *p)
     ap_destroy_pool(sp);
     if (bConflict)
         ssl_log(sm, SSL_LOG_WARN,
-                "Init: You cannot use name-based virtual hosts in conjunction with SSL!!");
+                "Init: You should not use name-based virtual hosts in conjunction with SSL!!");
 
     return;
 }
