@@ -1,3 +1,6 @@
+#ifndef lint
+static char const ssl_expr_yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
+#endif
 #include <stdlib.h>
 #define YYBYACC 1
 #define YYMAJOR 1
